@@ -28,15 +28,19 @@ function(x,fmt,...) {
   
   xx <- coredata(x, fmt)
   if(length(xx) == 0) {
-    cat('Data:\n')
-    print(numeric(0))
-    cat('\n')
-    cat('Index:\n')
-    index <- index(x)
-    if(length(index) == 0) {
-      print(index)
+    if(!is.null(colnames(x))) {
+      print(structure(structure(NULL,dim=c(0,NCOL(x))),dimnames=list(NULL,colnames(x))))
     } else {
-      str(index(x))
+      cat('Data:\n')
+      print(numeric(0))
+      cat('\n')
+      cat('Index:\n')
+      index <- index(x)
+      if(length(index) == 0) {
+        print(index)
+      } else {
+        str(index(x))
+      }
     }
   } else print(xx)
 }
