@@ -27,17 +27,18 @@ function(key,  vec, start=TRUE) {
   # TRUE return the _next_ observation (e.g. 20070801 if
   # missing would return 20070802 if available
   # FALSE would return the _previous_ obs.
-
   found <- FALSE
   lo <- 1; hi <- length(vec);
   rec <- NULL
 
   while(hi >= lo) {
     mid <- round((lo + hi) / 2)
-    if(key < vec[mid]) {
+    if(mid == 0)
+      return(NA)
+    if(mid != 0 && key < vec[mid]) {
       hi <- mid - 1
     } else 
-    if( key > vec[mid] ) {
+    if(mid != 0 && key > vec[mid]) {
       lo <- mid + 1
     } else {
       found <- TRUE
@@ -61,3 +62,13 @@ function(key,  vec, start=TRUE) {
   } else rec
 }
 
+naCheck <- function(x, n=0) {
+  if(is.null(dim(x)[2])) {
+    NAs <- .Call("naCheck", x, TRUE)
+  } else NAs <- .Call("naCheck", rowSums(x), TRUE)
+  ret <- list()
+  ret$NAs <- NAs
+  ret$nonNA <- (1+NAs):NROW(x)
+  ret$beg <- n+NAs
+  invisible(ret)
+}
